/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
/**
 * The type of revertible commit.
 *
 * @internal
 */
export var RevertibleKind;
(function (RevertibleKind) {
    /** A typical local commit */
    RevertibleKind[RevertibleKind["Default"] = 0] = "Default";
    /** A revertible that is the result of an undo. */
    RevertibleKind[RevertibleKind["Undo"] = 1] = "Undo";
    /** A revertible that is the result of a redo. */
    RevertibleKind[RevertibleKind["Redo"] = 2] = "Redo";
    /**
     * A revertible that is the result of a rebase and should replace a previously generated revertible.
     * todo: improve error reporting in this case
     */
    RevertibleKind[RevertibleKind["Rebase"] = 3] = "Rebase";
})(RevertibleKind || (RevertibleKind = {}));
/**
 * The result of a revert operation.
 *
 * @internal
 */
export var RevertResult;
(function (RevertResult) {
    /** The revert was successful. */
    RevertResult[RevertResult["Success"] = 0] = "Success";
    /** The revert failed. */
    RevertResult[RevertResult["Failure"] = 1] = "Failure";
})(RevertResult || (RevertResult = {}));
/**
 * The result of a discard operation.
 *
 * @internal
 */
export var DiscardResult;
(function (DiscardResult) {
    /** The discard was successful. */
    DiscardResult[DiscardResult["Success"] = 0] = "Success";
    /** The discard failed. */
    DiscardResult[DiscardResult["Failure"] = 1] = "Failure";
})(DiscardResult || (DiscardResult = {}));
//# sourceMappingURL=revertible.js.map